(* Template to write your own non relational abstract domain. *)

(* To implement your own non relational abstract domain,
 * first give the type of its elements. It must contain Top and Bot. *)
type t = Top | Bot

(* a printing function (useful for debuging), *)
let fprint ff e =
  Format.fprintf ff
    (match e with
    | Top -> "top" | Bot -> "bot")
    
(* The order of the lattice: here we have only bot ≤ top. *)
let order x y = x = Bot || y = Top

(* and infimums of the lattice. *)
let top = Top
let bottom = Bot

(* All the functions below are safe overapproximations.
 * You can keep them as this in a first implementation,
 * then refine them only when you need it to improve
 * the precision of your analyses. *)

(* Join of the lattice *)
let join x y = match x, y with
  | _, _ -> top

(* Meet of the lattice *)
let meet x y = match x, y with
  | _, _ -> bottom


(* The representation in the abstract domain of an interval of integers [n1, n2] *)
let sem_itv n1 n2 = top

(* Abstract arithmetic operations in the domain *)
let sem_plus x y = top
let sem_minus x y = top
let sem_times x y = top
let sem_div x y = top

(* Given an abstract value t, return the value if we add the
   constraint that t must represent a strictly positive integer *)

let sem_guard = function
  | t -> t


(* No need to touch this. *)     
let widening = join
let backsem_plus x y r = x, y
let backsem_minus x y r = x, y
let backsem_times x y r = x, y
let backsem_div x y r = x, y
